@@ -124,7 +124,9 @@ typedef struct {
 	void (*hid_update_hide_cb)(rnd_hid_attribute_t *attrib, void *hid_wdata);
 
 	/* Spare: see doc/developer/spare.txt */
-	void (*spare_f1)(void), (*spare_f2)(void);
+	int (*hid_get_selected_multi_cb)(rnd_hid_attribute_t *attrib, void *hid_ctx, vtp0_t *dst); /* optional: appends 0, 1 or more (rnd_hid_row_t *)'s in dst; returns 0 if any selection is added, -1 on error or no selection */
+
+	void (*spare_f2)(void);
 	long spare_l1, spare_l2, spare_l3, spare_l4;
 	void *spare_p1, *spare_p2, *spare_p3, *spare_p4;
 	double spare_d1, spare_d2;
@@ -178,7 +180,9 @@ typedef struct {
 	void (*free)(rnd_hid_attribute_t *attrib); /* called by DAD on free'ing the RND_HATT_BEGIN_COMPOUND and RND_HATT_END_COMPOUND widget */
 
 	/* Spare: see doc/developer/spare.txt */
-	void (*spare_f1)(void), (*spare_f2)(void);
+	int (*widget_focus)(rnd_hid_attribute_t *end, void *hid_ctx, int idx);
+
+	void (*spare_f2)(void);
 	long spare_l1, spare_l2, spare_l3, spare_l4;
 	void *spare_p1, *spare_p2, *spare_p3, *spare_p4;
 	double spare_d1, spare_d2;
@@ -307,14 +311,14 @@ do { \
 	RND_DAD_SET_ATTR_FIELD(table, name, rnd_strdup_printf printf_args); \
 } while(0)
 
-#define RND_DAD_STRING_SELECT_REGION(table, idx, first_char_offs, len) \
+#define RND_DAD_STRING_SELECT_REGION(dlg_hid_ctx, idx, first_char_offs, len) \
 do { \
 	if (rnd_gui->attr_dlg_widget_poke != NULL) { \
 		fgw_arg_t __args__[3]; \
 		__args__[0].type = FGW_STR; __args__[0].val.cstr = "select"; \
 		__args__[1].type = FGW_INT; __args__[1].val.nat_int = first_char_offs; \
 		__args__[2].type = FGW_INT; __args__[2].val.nat_int = len; \
-		rnd_gui->attr_dlg_widget_poke(table, idx, 3, __args__); \
+		rnd_gui->attr_dlg_widget_poke(dlg_hid_ctx, idx, 3, __args__); \
 	} \
 } while(0)
 
