@@ -1170,11 +1170,6 @@ static void redraw_region(rnd_design_t *hidlib, GdkRectangle *rect)
 	ctx.view.X2 = MAX(Px(priv->clip_rect.x), Px(priv->clip_rect.x + priv->clip_rect.width + 1));
 	ctx.view.Y2 = MAX(Py(priv->clip_rect.y), Py(priv->clip_rect.y + priv->clip_rect.height + 1));
 
-	ctx.view.X1 = ctx.view.X1;
-	ctx.view.X2 = ctx.view.X2;
-	ctx.view.Y1 = ctx.view.Y1;
-	ctx.view.Y2 = ctx.view.Y2;
-
 	eleft   = Vx(hidlib->dwg.X1);
 	eright  = Vx(hidlib->dwg.X2);
 	etop    = Vy(hidlib->dwg.Y1);
@@ -1211,6 +1206,8 @@ static void redraw_region(rnd_design_t *hidlib, GdkRectangle *rect)
 
 	ghid_gdk_draw_bg_image(hidlib);
 
+	ctx.coord_per_pix = ghidgui->port.view.coord_per_px;
+
 	rnd_app.expose_main(&gtk2_gdk_hid, &ctx, NULL);
 	ghid_gdk_draw_grid(hidlib);
 
@@ -1523,9 +1520,9 @@ static gboolean ghid_gdk_preview_expose(GtkWidget *widget, rnd_gtk_expose_t *ev,
 	xz = vw / (double)allocation.width;
 	yz = vh / (double)allocation.height;
 	if (xz > yz)
-		ghidgui->port.view.coord_per_px = xz;
+		ctx->coord_per_pix = ghidgui->port.view.coord_per_px = xz;
 	else
-		ghidgui->port.view.coord_per_px = yz;
+		ctx->coord_per_pix = ghidgui->port.view.coord_per_px = yz;
 
 	priv->base_pixel = priv->out_pixel = window;
 	ghidgui->port.view.canvas_width = allocation.width;
