@@ -22,8 +22,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 #ifndef HID_GL_DRAW_GL_H
@@ -55,7 +55,7 @@ struct hidgl_draw_s {
 	void (*set_color)(GLfloat r, GLfloat g, GLfloat b, GLfloat a);
 	void (*flush)(void);
 	void (*reset)(void);
-	void (*expose_init)(int w, int h, const rnd_color_t *bg_c);
+	void (*expose_init)(int x0, int y0, int w, int h, const rnd_color_t *bg_c);
 
 	/* set up transformation to translate tx,ty and zoom zx,zy,zz */
 	void (*set_view)(double tx, double ty, double zx, double zy, double zz);
