@@ -21,13 +21,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,8 +35,8 @@
 #include <math.h>
 #include <assert.h>
 
-#include <librnd/core/grid.h>
-#include <librnd/core/hid.h>
+#include <librnd/hid/grid.h>
+#include <librnd/hid/hid.h>
 #include <librnd/poly/rtree.h>
 #include <librnd/core/hidlib.h>
 
@@ -265,7 +265,7 @@ RND_INLINE void reserve_grid_points(int n, int n3)
 	}
 }
 
-void hidgl_draw_local_grid(rnd_hidlib_t *hidlib, rnd_coord_t grd, rnd_coord_t cx, rnd_coord_t cy, int radius, double scale, rnd_bool cross_grid)
+void hidgl_draw_local_grid(rnd_design_t *hidlib, rnd_coord_t grd, rnd_coord_t cx, rnd_coord_t cy, int radius, double scale, rnd_bool cross_grid)
 {
 	int npoints = 0;
 	rnd_coord_t x, y;
@@ -309,17 +309,17 @@ void hidgl_draw_local_grid(rnd_hidlib_t *hidlib, rnd_coord_t grd, rnd_coord_t cx
 	hidgl_draw.xor_end();
 }
 
-void hidgl_draw_grid(rnd_hidlib_t *hidlib, rnd_coord_t grd, rnd_box_t *drawn_area, double scale, rnd_bool cross_grid)
+void hidgl_draw_grid(rnd_design_t *hidlib, rnd_coord_t grd, rnd_box_t *drawn_area, double scale, rnd_bool cross_grid)
 {
 	rnd_coord_t x1, y1, x2, y2, n, i, n3;
 	double x, y;
 
 	hidgl_draw.xor_start();
 
-	x1 = rnd_grid_fit(MAX(0, drawn_area->X1), grd, hidlib->grid_ox);
-	y1 = rnd_grid_fit(MAX(0, drawn_area->Y1), grd, hidlib->grid_oy);
-	x2 = rnd_grid_fit(MIN(hidlib->size_x, drawn_area->X2), grd, hidlib->grid_ox);
-	y2 = rnd_grid_fit(MIN(hidlib->size_y, drawn_area->Y2), grd, hidlib->grid_oy);
+	x1 = rnd_grid_fit(MAX(hidlib->dwg.X1, drawn_area->X1), grd, hidlib->grid_ox);
+	y1 = rnd_grid_fit(MAX(hidlib->dwg.Y1, drawn_area->Y1), grd, hidlib->grid_oy);
+	x2 = rnd_grid_fit(MIN(hidlib->dwg.X2, drawn_area->X2), grd, hidlib->grid_ox);
+	y2 = rnd_grid_fit(MIN(hidlib->dwg.Y2, drawn_area->Y2), grd, hidlib->grid_oy);
 
 	if (x1 > x2) {
 		rnd_coord_t tmp = x1;
@@ -783,9 +783,9 @@ void hidgl_fill_polygon_offs(int n_coords, rnd_coord_t *x, rnd_coord_t *y, rnd_c
 	free(vertices);
 }
 
-void hidgl_expose_init(int w, int h, const rnd_color_t *bg_c)
+void hidgl_expose_init(int x0, int y0, int w, int h, const rnd_color_t *bg_c)
 {
-	hidgl_draw.expose_init(w, h, bg_c);
+	hidgl_draw.expose_init(x0, y0, w, h, bg_c);
 }
 
 void hidgl_draw_crosshair(rnd_coord_t x, rnd_coord_t y, float red, float green, float blue, rnd_coord_t minx, rnd_coord_t miny, rnd_coord_t maxx, rnd_coord_t maxy)
