@@ -143,7 +143,7 @@ static int pa_coll_jump(rnd_vnode_t **cur, pa_direction_t *cdir, pa_jump_rule_t
 
 	/* cross-vertex means we are at an intersection and have to decide which
 	   edge to continue at */
-	DEBUG_JUMP("jump entering node at %$mD\n", (*cur)->point[0], (*cur)->point[1]);
+	DEBUG_JUMP_COORDS("jump entering node", (*cur), "");
 
 	start = d = (*cdir == PA_FORWARD) ? (*cur)->cvclst_prev->prev : (*cur)->cvclst_next->prev;
 	do {
@@ -156,7 +156,7 @@ static int pa_coll_jump(rnd_vnode_t **cur, pa_direction_t *cdir, pa_jump_rule_t
 		if (!e->flg.mark && rule(d->poly, e, &newdir)) {
 			if (((d->side == 'N') && (newdir == PA_FORWARD)) || ((d->side == 'P') && (newdir == PA_BACKWARD))) {
 				rnd_vnode_t *nnd = (newdir == PA_FORWARD) ? e->next : e;
-				DEBUG_JUMP("jump leaving node at %$mD\n", nnd->point[0], nnd->point[1]);
+				DEBUG_JUMP_COORDS("jump leaving node", nnd, "");
 
 				*cur = d->parent;
 				*cdir = newdir;
@@ -168,7 +168,8 @@ static int pa_coll_jump(rnd_vnode_t **cur, pa_direction_t *cdir, pa_jump_rule_t
 	return rnd_false;
 }
 
-RND_INLINE int pa_is_node_coords_integer(rnd_vnode_t *nd)
+/* Returns 1 if bigcoord is enabled and nd is not on an exact integer coordinate */
+RND_INLINE int pa_is_node_coords_non_integer(rnd_vnode_t *nd)
 {
 /* If we have an intersection, we have a high resolution coord; if that coord
    is not integer, we had to do a rounding to get the output integer coord */
@@ -187,7 +188,10 @@ RND_INLINE int pa_is_node_coords_integer(rnd_vnode_t *nd)
 /* This is Collect() in the original paper */
 RND_INLINE int pa_coll_gather(rnd_vnode_t *start, rnd_pline_t **result, pa_jump_rule_t v_rule, pa_direction_t dir)
 {
-	rnd_vnode_t *nd, *newnd;
+	rnd_vnode_t *nd, *prev_nd, *first_nd; /* these are in terms of input node (high precision) */
+	rnd_vnode_t *newnd; /* this is output node, always rounded to integer */
+	int risky = 0, orig_dir;
+	pa_big2_coord_t big_area = {0};
 
 	DEBUG_GATHER("gather direction = %d\n", dir);
 	*result = NULL;
@@ -201,17 +205,25 @@ RND_INLINE int pa_coll_gather(rnd_vnode_t *start, rnd_pline_t **result, pa_jump_
 			if (*result == NULL)
 				return pa_err_no_memory;
 			newnd = (*result)->head;
-			newnd->flg.rounded = nd->flg.rounded | pa_is_node_coords_integer(nd);
+			prev_nd = first_nd = nd;
 		}
 		else { /* insert subsequent */
 			newnd = rnd_poly_node_create(nd->point);
 			if (newnd == NULL)
 				return pa_err_no_memory;
-			newnd->flg.rounded = nd->flg.rounded | pa_is_node_coords_integer(nd);
 			rnd_poly_vertex_include((*result)->head->prev, newnd);
+			pa_big_area_incremental(big_area, nd, prev_nd);
+			prev_nd = nd;
 		}
 
-		DEBUG_GATHER("gather vertex at %$mD\n", nd->point[0], nd->point[1]);
+		if (nd->cvclst_prev != NULL) {
+			DEBUG_GATHER_COORDS("gather vertex", nd, "(risk)");
+			newnd->flg.risk = 1;
+			risky++;
+		}
+		else {
+			DEBUG_GATHER_COORDS("gather vertex", nd, "");
+		}
 
 		/* mark the edge as included; mark both if SHARED edge */
 		newnd = (dir == PA_FORWARD) ? nd : nd->prev;
@@ -220,6 +232,31 @@ RND_INLINE int pa_coll_gather(rnd_vnode_t *start, rnd_pline_t **result, pa_jump_
 			newnd->shared->flg.mark = 1;
 	}
 
+	if (*result == NULL)
+		return pa_err_ok;  /* test case: gixedf */
+
+
+/*	orig_dir = pa_pline_update_big2small(*result);
+	rnd_trace("Bad orig=%d ", orig_dir);*/
+	pa_pline_update(*result, 1);
+	pa_big_area_incremental(big_area, first_nd, prev_nd);
+	orig_dir = pa_big_area2ori(big_area);
+
+#ifdef PA_BIGCOORD_ISC
+	/* triangle flip special case; test case: fixedd, fixedi. The implicit rounding when
+	   nd is copied to newnd and cvc high resolution isc is dropped may cause
+	   self intersections in polygons that have more than 3 corners (this is
+	   handled later in postproc). In a small triangle such rounding error
+	   introduced change can not cause self intersect but can flip direction
+	   if one corner jumps over an edge. If that happens we can not "unround"
+	   the output triangle, the closest thing we can do is to invert it. */
+	if (risky && ((*result)->Count == 3)) {
+/*		rnd_trace(" triangle flip: [%d] orig_dir=%d res_dir=%d flipped=%d\n", (*result)->Count, orig_dir, (*result)->flg.orient, ((*result)->flg.orient != orig_dir));*/
+		if ((*result)->flg.orient != orig_dir)
+			pa_pline_invert(*result);
+	}
+#endif
+
 	return pa_err_ok;
 }
 
@@ -230,7 +267,7 @@ RND_INLINE void pa_collect_gather(jmp_buf *e, rnd_vnode_t *cur, pa_direction_t d
 	int res;
 
 	/* gather a polyline in pl */
-	vn = (dir == PA_FORWARD) ? cur : cur->next;
+	vn = (dir == PA_FORWARD) ? cur : cur->prev;
 	res = pa_coll_gather(vn, &pl, j_rule, dir);
 	if (res != pa_err_ok) {
 		pa_pline_free(&pl);
@@ -240,7 +277,6 @@ RND_INLINE void pa_collect_gather(jmp_buf *e, rnd_vnode_t *cur, pa_direction_t d
 	if (pl == NULL)
 		return;
 
-	pa_pline_update(pl, rnd_true);
 
 	if (pl->Count > 2) {
 		DEBUG_GATHER("adding contour with %d vertices and direction %c\n", pl->Count, (pl->flg.orient ? 'F' : 'B'));
