@@ -36,18 +36,38 @@ typedef rnd_vertex_t rnd_vector_t;
 
 extern const rnd_vector_t rnd_vect_zero;
 
+typedef enum {
+	RND_VNODE_LINE = 0, /* there is no ->curve */
+	RND_VNODE_ARC
+} rnd_vnode_curve_type_t;
+
 typedef struct pa_conn_desc_s pa_conn_desc_t; /* A "descriptor" that makes up a "connectivity list" in the paper */
 typedef struct rnd_vnode_s rnd_vnode_t;
 struct rnd_vnode_s {
 	rnd_vnode_t *next, *prev, *shared;
 	struct {
-		unsigned int plabel:3; /* one of pa_plinept_label_t; for the edge {v to v->next} (the edge forward of v) */
-		unsigned int mark:1;
-		unsigned int in_hub:1;
-		unsigned int rounded:1; /* had high resolution coords that got rounded for the output; kept track until the bool_postprocess step that resets this. */
+		unsigned plabel:3;     /* one of pa_plinept_label_t; for the edge {v to v->next} (the edge forward of v) */
+		unsigned mark:1;
+		unsigned in_hub:1;
+		unsigned risk:1;       /* the vertex was an intersection so there is a risk of topology change (line jumping over another vertex); kept track until the bool_postprocess step that handles the risk and resets this. */
+		unsigned start:1;      /* used in pa_self_isc: the node the loop search has started from */
+		unsigned curve_type:4; /* one of rnd_vnode_curve_type_t */
 	} flg;
 	pa_conn_desc_t *cvclst_prev, *cvclst_next; /* "cross vertex connection list" */
-	rnd_vector_t point;
+	rnd_vector_t point; /* startpoint; endpoint is ->next.point */
+
+	/* fields above are binary-compatible with the old poly lib */
+
+	union {
+		struct { /* circular arc */
+			rnd_vector_t center; /* part of the specification: saved */
+			/* cached fields (not saved) */
+			rnd_ucoord_t r[6]; /* pa_big_coord_t really */
+			/* angle of point from the center [0..4); pa_big_coord_t aim = aim_int.aim_frac */
+			rnd_ucoord_t aim_frac[2]; /* fraction part of aim ("angle") for start and endpoint */
+			char aim_int[2]; /* integer part of the aim ("angle") for start and endpoint */
+		} arc;
+	} curve;
 };
 
 typedef struct rnd_pline_s rnd_pline_t;
@@ -67,6 +87,7 @@ struct rnd_pline_s {
 	struct {
 		unsigned int llabel:3; /* one of pa_pline_label_t */
 		unsigned int orient:1;
+		unsigned int risky:1;  /* need to run selfisc resolver in postproc */
 	} flg;
 };
 
@@ -107,10 +128,14 @@ struct rnd_polyarea_s {
 	rnd_polyarea_t *f, *b;
 	rnd_pline_t *contours;
 	rnd_rtree_t *contour_tree;
+
+	/* internal */
+	unsigned from_selfisc:1; /* postproc is running from within selfisc - avoid infinite loops */
 };
 
 rnd_bool rnd_polyarea_alloc_copy(rnd_polyarea_t **dst, const rnd_polyarea_t *srcfst);
 void rnd_polyarea_m_include(rnd_polyarea_t **list, rnd_polyarea_t *a);
+void pa_polyarea_unlink(rnd_polyarea_t **list, rnd_polyarea_t *island);
 
 /* Allocate a new polyarea and copy src into it; returns NULL on error */
 rnd_polyarea_t *rnd_polyarea_dup(const rnd_polyarea_t *src);
@@ -128,19 +153,36 @@ rnd_bool pa_polyarea_copy_plines(rnd_polyarea_t *dst, const rnd_polyarea_t *src)
    in a pa that doesn't have an outer. */
 rnd_bool pa_polyarea_insert_pline(rnd_polyarea_t *pa, rnd_pline_t *pl);
 
-rnd_bool rnd_polyarea_contour_exclude(rnd_polyarea_t *p, rnd_pline_t *c);
+/* Remove pl from pa (but do not free pl) */
+void pa_polyarea_del_pline(rnd_polyarea_t *pa, rnd_pline_t *pl);
 
 
 rnd_bool rnd_polyarea_contour_check(rnd_pline_t *a);
 
+/* Make sure pa is not self intersecting; resolves any type of self intersection
+   by fixing pa. Returns number of changes made */
+rnd_cardinal_t rnd_polyarea_split_selfisc(rnd_polyarea_t **pa);
+
+
 rnd_bool rnd_polyarea_contour_inside(rnd_polyarea_t *c, rnd_vector_t v0);
 rnd_bool rnd_polyarea_touching(rnd_polyarea_t *p1, rnd_polyarea_t *p2);
 
+/* Returns whether two islands intersect or not. Only the first island of a and
+   first island of b are verified. They are considered intersecting if and only
+   if there's a vertex of a inside of b or a vertex of b inside of a. Touching
+   on the edges is not considered intersection here. */
+rnd_bool rnd_polyarea_island_isc(const rnd_polyarea_t *a, const rnd_polyarea_t *b);
+
 /*** tools for clipping ***/
 
 /* checks whether point lies within contour independently of its orientation */
 int pa_pline_is_point_inside(const rnd_pline_t *c, rnd_vector_t v);
 
+/* same as pa_pline_is_point_inside() but uses sub-nanometer resolution and
+   is always accurate */
+int pa_pline_is_node_inside(const rnd_pline_t *pl, rnd_vnode_t nd);
+
+
 /* Return whether inner is within outer. Caller must ensure they are not
    intersecting. Returns the correct result if pl and inner share common
    points on contour. (Identical contours are treated as being inside each
@@ -153,9 +195,16 @@ rnd_polyarea_t *pa_polyarea_alloc(void);
 void pa_polyarea_free_all(rnd_polyarea_t **p);
 void pa_polyarea_init(rnd_polyarea_t *p); /* reset/initialize p's fields (call after malloc) */
 void rnd_poly_plines_free(rnd_pline_t **pl);
+
+/* Verify that the polygon island is valid (not self intersecting); returns
+   true if valid. */
+rnd_bool rnd_poly_valid_island(rnd_polyarea_t *p);
+
+/* Verify that the polygon is valid (not self intersecting); checks all islands;
+   returns true if valid. */
 rnd_bool rnd_poly_valid(rnd_polyarea_t *p);
 
-enum rnd_poly_bool_op_e {
+typedef enum rnd_poly_bool_op_e {
 	RND_PBO_UNITE,
 	RND_PBO_ISECT,
 	RND_PBO_SUB,
@@ -234,11 +283,7 @@ void rnd_polyarea_get_tree_seg(void *obj, rnd_coord_t *x1, rnd_coord_t *y1, rnd_
 /* create a (rnd_rtree_t *) of each segment derived from a contour object of src */
 void *rnd_poly_make_edge_tree(rnd_pline_t *src);
 
-
-/* EPSILON^2 for endpoint matching; the bool algebra code is not
-   perfect and causes tiny self intersections at the end of sharp
-   spikes. Accept at most 10 nanometer of such intersection */
-#define RND_POLY_ENDP_EPSILON 100
+int pa_pline_is_vnode_inside(const rnd_pline_t *pl, const rnd_vnode_t *nd, int point_on_edge_is_in);
 
 /*** generic geo ***/
 int rnd_point_in_triangle(rnd_vector_t A, rnd_vector_t B, rnd_vector_t C, rnd_vector_t P);
